@@ -1,5 +1,13 @@
 #!/usr/bin/env python3
-"""The BluePill Simulation."""
+"""The BluePill Simulation.
+
+In the BluePill simulation
+agents choose change their state in sequence:
+rock -> paper -> scissors -> rock.
+The starting state of every agent is chosen at random.
+State of all agents is stored in a single SQLite3 state store.
+Agent objects themselves store not state information.
+"""
 
 import random
 import logging
@@ -23,28 +31,34 @@ from matrixabm import (
     Constructor,
 )
 
-# Standard Actor IDs
+
+# Actor IDs
 AID_MAIN = "main"
-AID_POPULATION = "population"
+AID_SIMULATOR = "simulator"
 AID_COORDINATOR = "coordinator"
 AID_RUNNER = "runner"
+AID_POPULATION = "population"
 AID_TIMESTEP_GEN = "timestep_gen"
 AID_SUMMARY_WRITER = "summary_writer"
 AID_SQLITE3 = "sqlite3"
+
+# The database schema name
 STORE_NAME = "bluepill"
 
 
 class BluePillStore(SQLite3Store):
-    """The BluePill State Store."""
+    """The BluePill Store.
 
-    def __init__(self, store_name, connector_name):
+    The BluePill store maintains the state of the simulation.
+    The store object is a SQLite3 file.
+    The file contains one table called "state".
+    """
+
+    def __init__(self):
         """Initialize."""
-        super().__init__(store_name, AID_MAIN, connector_name)
+        super().__init__(STORE_NAME, AID_SIMULATOR, AID_SQLITE3)
 
-        self.setup()
-
-    def setup(self):
-        """Setup the state table."""
+        # Setup the state table
         con = asys.local_actor(AID_SQLITE3).connection
         sql = f"""
             create table if not exists
@@ -56,14 +70,38 @@ class BluePillStore(SQLite3Store):
         con.execute(sql)
 
     def set_state(self, agent_id, state, step):
-        """Set the agent state."""
+        """Set the agent state.
+
+        Parameters
+        ----------
+        agent_id : str
+            ID of the agent
+        state : str
+            State of the agent
+            One of "rock", "paper", and "scissors"
+        step : int
+            The current timestep
+        """
         con = asys.local_actor(AID_SQLITE3).connection
         sql = f"insert into {self.store_name}.state values (?,?,?)"
         con.execute(sql, (agent_id, state, step))
 
     @staticmethod
     def get_state(store_name, agent_id):
-        """Get the state of the agent."""
+        """Get the latest state of the agent.
+
+        Parameters
+        ----------
+        store_name : str
+            Name of the SQLite3 database
+        agent_id : str
+            ID of the agent
+
+        Returns
+        -------
+        str or None
+            The latest state of the given agent.
+        """
         con = asys.local_actor(AID_SQLITE3).connection
         sql = f"""
             select state
@@ -74,18 +112,24 @@ class BluePillStore(SQLite3Store):
             """
         row = con.execute(sql, (agent_id,)).fetchone()
         if row is None:
-            return random.choice(["rock", "paper", "scissors"])
-        else:
-            return row[0]
+            return None
+
+        return row[0]
 
 
 class BluePillAgent(Agent):
-    """The BluePill Agent."""
+    """The BluePill Agent.
+
+    The BluePill agent has the following behavior.
+    At start its state is chosen at random from ["rock", "paper", and "scissors"]
+    Every following timestep its state cycles in the given order:
+    rock -> paper -> scissors -> rock.
+    """
 
     def __init__(self, store_name, agent_id):
         self.agent_id = agent_id
         self.store_name = store_name
-        self.state = BluePillStore.get_state(self.store_name, self.agent_id)
+        self.state = random.choice(["rock", "paper", "scissors"])
 
     def step(self, timestep):
         """Return the step update."""
@@ -122,7 +166,10 @@ class BluePillAgent(Agent):
 
 
 class BluePillPopulation(AgentPopulation):
-    """Blue Pill Agent Population."""
+    """Blue Pill Agent Population.
+
+    At every timestep between 100 and 200 new agents are created.
+    """
 
     def __init__(self):
         super().__init__(AID_COORDINATOR)
@@ -143,12 +190,27 @@ class BluePillPopulation(AgentPopulation):
         return ret
 
 
-class BluePillSimulator(Simulator):
-    """Blue Pill Simulator."""
+class Main:
+    """The main actor."""
 
     def __init__(self, store_path, summary_dir):
-        """Initialize."""
-        super().__init__(
+        """Initialize.
+
+        store_path : str
+            Path to the local sqlite3 file
+        summary_dir : str
+            Directory where runtime summary is to be written
+        """
+        self.store_path = store_path
+        self.summary_dir = summary_dir
+
+    def main(self):
+        """Setup the connectors on the ranks."""
+        # Create the simulator
+        asys.create_actor(
+            asys.MASTER_RANK,
+            AID_SIMULATOR,
+            Simulator,
             coordinator_aid=AID_COORDINATOR,
             runner_aid=AID_RUNNER,
             population_aid=AID_POPULATION,
@@ -156,60 +218,68 @@ class BluePillSimulator(Simulator):
             store_names=[STORE_NAME],
         )
 
-        self.store_path = store_path
-        self.summary_dir = summary_dir
-
-    def main(self):
-        """Setup the connectors on the ranks."""
-        for rank in asys.ranks():
-            asys.create_actor(
-                rank, AID_SQLITE3, SQLite3Manager, [STORE_NAME], [self.store_path]
-            )
-
-        asys.create_actor(
-            asys.MASTER_RANK, AID_TIMESTEP_GEN, RangeTimestepGenerator, 10
-        )
-        asys.create_actor(asys.MASTER_RANK, AID_POPULATION, BluePillPopulation)
-        asys.create_actor(
-            asys.MASTER_RANK, AID_SUMMARY_WRITER, TensorboardWriter, self.summary_dir
-        )
-
+        # Create the coordinator
         load_balancer = GreedyLoadBalancer(WORLD_SIZE)
         asys.create_actor(
             asys.MASTER_RANK,
             AID_COORDINATOR,
             Coordinator,
             load_balancer,
-            AID_MAIN,
+            AID_SIMULATOR,
             AID_RUNNER,
             AID_SUMMARY_WRITER,
         )
 
+        # Create the SQLite3 connection managers on every rank
+        for rank in asys.ranks():
+            asys.create_actor(
+                rank, AID_SQLITE3, SQLite3Manager, [STORE_NAME], [self.store_path]
+            )
+
+        # Create a store on the first rank of every node
         store_ranks = {STORE_NAME: []}
         for node in asys.nodes():
             rank = asys.node_ranks(node)[0]
             store_ranks[STORE_NAME].append(rank)
-
         store_proxies = {
             name: asys.ActorProxy(store_ranks[name], STORE_NAME)
             for name, ranks in store_ranks.items()
         }
-        store_proxies[STORE_NAME].create_actor_(BluePillStore, STORE_NAME, AID_SQLITE3)
+        store_proxies[STORE_NAME].create_actor_(BluePillStore)
 
+        # Create the runners on every rank
         for rank in asys.ranks():
             asys.create_actor(
                 rank, AID_RUNNER, Runner, store_proxies, AID_COORDINATOR, AID_RUNNER
             )
 
-        asys.ActorProxy(asys.MASTER_RANK, AID_MAIN).start()
+        # Create the timestep generator
+        asys.create_actor(
+            asys.MASTER_RANK, AID_TIMESTEP_GEN, RangeTimestepGenerator, 10
+        )
+
+        # Create the agent population
+        asys.create_actor(asys.MASTER_RANK, AID_POPULATION, BluePillPopulation)
+
+        # Create the tensorboard summary directory manager
+        asys.create_actor(
+            asys.MASTER_RANK, AID_SUMMARY_WRITER, TensorboardWriter, self.summary_dir
+        )
+
+        # Start the simulator
+        asys.ActorProxy(asys.MASTER_RANK, AID_SIMULATOR).start()
 
 
 @click.command()
 @click.argument("store_path")
 @click.argument("summary_dir")
 def main(store_path, summary_dir):
-    """Run the simulation."""
-    asys.start(AID_MAIN, BluePillSimulator, store_path, summary_dir)
+    """Run a Blue Pill simulation.
+
+    STORE_PATH should point the file when the SQLite3 store file should be created.
+    SUMMARY_DIR should point to the directory where runtime summary is to be written.
+    """
+    asys.start(AID_MAIN, Main, store_path, summary_dir)
 
 
 if __name__ == "__main__":
